; The CMD file.
;
; Two parts: 1. Command definition and  2. State entry
; (state entry is after the commands def section)
;
; 1. Command definition
; ---------------------
; Note: The commands are CASE-SENSITIVE, and so are the command names.
; The eight directions are:
;   B, DB, D, DF, F, UF, U, UB     (all CAPS)
;   corresponding to back, down-back, down, downforward, etc.
; The six buttons are:
;   a, b, c, x, y, z               (all lower case)
;   In default key config, abc are are the bottom, and xyz are on the
;   top row. For 2 button characters, we recommend you use a and b.
;   For 6 button characters, use abc for kicks and xyz for punches.
;
; Each [Command] section defines a command that you can use for
; state entry, as well as in the CNS file.
; The command section should look like:
;
;   [Command]
;   name = some_name
;   command = the_command
;   time = time (optional)
;   buffer.time = time (optional)
;
; - some_name
;   A name to give that command. You'll use this name to refer to
;   that command in the state entry, as well as the CNS. It is case-
;   sensitive (QCB_a is NOT the same as Qcb_a or QCB_A).
;
; - command
;   list of buttons or directions, separated by commas. Each of these
;   buttons or directions is referred to as a "symbol".
;   Directions and buttons can be preceded by special characters:
;   slash (/) - means the key must be held down
;          egs. command = /D       ;hold the down direction
;               command = /DB, a   ;hold down-back while you press a
;   tilde (~) - to detect key releases
;          egs. command = ~a       ;release the a button
;               command = ~D, F, a ;release down, press fwd, then a
;          If you want to detect "charge moves", you can specify
;          the time the key must be held down for (in game-ticks)
;          egs. command = ~30a     ;hold a for at least 30 ticks, then release
;   dollar ($) - Direction-only: detect as 4-way
;          egs. command = $D       ;will detect if D, DB or DF is held
;               command = $B       ;will detect if B, DB or UB is held
;   plus (+) - Buttons only: simultaneous press
;          egs. command = a+b      ;press a and b at the same time
;               command = x+y+z    ;press x, y and z at the same time
;   greater-than (>) - means there must be no other keys pressed or released
;                      between the previous and the current symbol.
;          egs. command = a, >~a   ;press a and release it without having hit
;                                  ;or released any other keys in between
;   You can combine the symbols:
;     eg. command = ~30$D, a+b     ;hold D, DB or DF for 30 ticks, release,
;                                  ;then press a and b together
;
;   Note: Successive direction symbols are always expanded in a manner similar
;         to this example:
;           command = F, F
;         is expanded when MUGEN reads it, to become equivalent to:
;           command = F, >~F, >F
;
;   It is recommended that for most "motion" commads, eg. quarter-circle-fwd,
;   you start off with a "release direction". This makes the command easier
;   to do.
;
; - time (optional)
;   Time allowed to do the command, given in game-ticks. The default
;   value for this is set in the [Defaults] section below. A typical
;   value is 15.
;
; - buffer.time (optional)
;   Time that the command will be buffered for. If the command is done
;   successfully, then it will be valid for this time. The simplest
;   case is to set this to 1. That means that the command is valid
;   only in the same tick it is performed. With a higher value, such
;   as 3 or 4, you can get a "looser" feel to the command. The result
;   is that combos can become easier to do because you can perform
;   the command early. Attacks just as you regain control (eg. from
;   getting up) also become easier to do. The side effect of this is
;   that the command is continuously asserted, so it will seem as if
;   you had performed the move rapidly in succession during the valid
;   time. To understand this, try setting buffer.time to 30 and hit
;   a fast attack, such as KFM's light punch.
;   The default value for this is set in the [Defaults] section below. 
;   This parameter does not affect hold-only commands (eg. /F). It
;   will be assumed to be 1 for those commands.
;
; If you have two or more commands with the same name, all of them will
; work. You can use it to allow multiple motions for the same move.
;
; Some common commands examples are given below.
;
; [Command] ;Quarter circle forward + x
; name = "QCF_x"
; command = ~D, DF, F, x
;
; [Command] ;Half circle back + a
; name = "HCB_a"
; command = ~F, DF, D, DB, B, a
;
; [Command] ;Two quarter circles forward + y
; name = "2QCF_y"
; command = ~D, DF, F, D, DF, F, y
;
; [Command] ;Tap b rapidly
; name = "5b"
; command = b, b, b, b, b
; time = 30
;
; [Command] ;Charge back, then forward + z
; name = "charge_B_F_z"
; command = ~60$B, F, z
; time = 10
;
; [Command] ;Charge down, then up + c
; name = "charge_D_U_c"
; command = ~60$D, U, c
; time = 10


;-| Button Remapping |-----------------------------------------------------
; This section lets you remap the player's buttons (to easily change the
; button configuration). The format is:
;   old_button = new_button
; If new_button is left blank, the button cannot be pressed.
[Remap]
x = x
y = y
z = z
a = a
b = b
c = c
s = s

;-| Default Values |-------------------------------------------------------
[Defaults]
; Default value for the "time" parameter of a Command. Minimum 1.
command.time = 15

; Default value for the "buffer.time" parameter of a Command. Minimum 1,
; maximum 30.
command.buffer.time = 1


;-| Super Motions |--------------------------------------------------------
;The following two have the same name, but different motion.
;Either one will be detected by a "command = TripleKFPalm" trigger.
;Time is set to 20 (instead of default of 15) to make the move
;easier to do.
;

[Command]
name = "RainingMutants"
command = ~D, DF, F, D, DB, B, x
time = 20

[Command]
name = "RainingMutants"
command = ~D, DF, F, D, DB, B, y
time = 20

[Command]
name = "Puncture"
command = ~D, DF, F, D, DF, F, x
time = 20

[Command]
name = "Puncture"   ;Same name as above
command = ~D, DF, F, D, DF, F, y
time = 20

[Command]
name = "HwaJai"
command = ~D, DB, B, D, DF, F, x
time = 20

[Command]
name = "HwaJai"   ;Same name as above
command = ~D, DB, B, D, DF, F, y
time = 20

[Command]
name = "Cs"
command = ~D, DF, F, D, DF, F, a
time = 20

[Command]
name = "CloneSuper"   ;Same name as above
command = ~D, DF, F, D, DF, F, b
time = 20

;-| Special Motions |------------------------------------------------------
name = "kick"
command =  F, F, a

[Command]
name = "kick"
command =  F, F, b

[Command]
name = "QCB_x"
command = ~D, DB, B, x

[Command]
name = "QCB_y"
command = ~D, DB, B, y

[Command]
name = "QCB_z"
command = ~D, DB, B, z

[Command]
name = "light kung fu palm"
command = ~D, DB, B, x

[Command]
name = "light kung fu palm"
command = ~D, DB, B, y

[Command]
name = "BlenderEdge"
command = F, F, x

[Command]
name = "BlenderEdge"
command = F, F, y

;-| Double Tap |-----------------------------------------------------------
[Command]
name = "FF"     ;Required (do not remove)
command = F, F
time = 10

[Command]
name = "BB"     ;Required (do not remove)
command = B, B
time = 10

;-| 2/3 Button Combination |-----------------------------------------------
[Command]
name = "recovery";Required (do not remove)
command = x+y
time = 1

;-| Dir + Button |---------------------------------------------------------
[Command]
name = "down_a"
command = /$D,a
time = 1

[Command]
name = "down_b"
command = /$D,b
time = 1

;-| Single Button |---------------------------------------------------------
[Command]
name = "a"
command = a
time = 1

[Command]
name = "b"
command = b
time = 1

[Command]
name = "c"
command = c
time = 1

[Command]
name = "x"
command = x
time = 1

[Command]
name = "y"
command = y
time = 1

[Command]
name = "z"
command = z
time = 1

[Command]
name = "start"
command = s
time = 1

;-| Hold Dir |--------------------------------------------------------------
[Command]
name = "holdfwd";Required (do not remove)
command = /$F
time = 1

[Command]
name = "holdback";Required (do not remove)
command = /$B
time = 1

[Command]
name = "holdup" ;Required (do not remove)
command = /$U
time = 1

[Command]
name = "holddown";Required (do not remove)
command = /$D
time = 1

;---------------------------------------------------------------------------
; 2. State entry
; --------------
; This is where you define what commands bring you to what states.
;
; Each state entry block looks like:
;   [State -1, Label]           ;Change Label to any name you want to use to
;                               ;identify the state with.
;   type = ChangeState          ;Don't change this
;   value = new_state_number
;   trigger1 = command = command_name
;   . . .  (any additional triggers)
;
; - new_state_number is the number of the state to change to
; - command_name is the name of the command (from the section above)
; - Useful triggers to know:
;   - statetype
;       S, C or A : current state-type of player (stand, crouch, air)
;   - ctrl
;       0 or 1 : 1 if player has control. Unless "interrupting" another
;                move, you'll want ctrl = 1
;   - stateno
;       number of state player is in - useful for "move interrupts"
;   - movecontact
;       0 or 1 : 1 if player's last attack touched the opponent
;                useful for "move interrupts"
;
; Note: The order of state entry is important.
;   State entry with a certain command must come before another state
;   entry with a command that is the subset of the first.
;   For example, command "fwd_a" must be listed before "a", and
;   "fwd_ab" should come before both of the others.
;
; For reference on triggers, see CNS documentation.
;
; Just for your information (skip if you're not interested):
; This part is an extension of the CNS. "State -1" is a special state
; that is executed once every game-tick, regardless of what other state
; you are in.


; Don't remove the following line. It's required by the CMD standard.
[Statedef -1]

;===========================================================================
;---------------------------------------------------------------------------
; Raining Mutants
[State -1, Raining Mutants]
type = ChangeState
value = 3313
triggerall = power >= 3000
triggerall = statetype != A
triggerall = RoundState = 2
trigger1 = command = "RainingMutants" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "RainingMutants" && AILevel = 0
trigger2 = hitdefattr = SC, NA, SA, HA
trigger2 = stateno != [3050,3100)
trigger2 = movecontact
trigger3 = Random < 0.5*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X > 120
trigger3 = ctrl && p2movetype = A
trigger4 = Random < 1*AILevel*AILevel && AILevel != 0
trigger4 = StateNo = 1010 && AnimElemNo(0) = 6 && MoveHit = 1

;---------------------------------------------------------------------------
; CloneSuper
[State -1, CloneSuper]
type = ChangeState
value = 3002
triggerall = power >= 2000
triggerall = RoundState = 2
trigger1 = command = "Cs" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "Cs" && AILevel = 0
trigger2 = hitdefattr = SC, NA, SA, HA
trigger2 = stateno != [3002,3003)
trigger2 = movecontact
trigger3 = Random < 0.25*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y > 120
trigger3 = ctrl && p2movetype = A

;---------------------------------------------------------------------------
; HwaJai
[State -1, HwaJai]
type = ChangeState
value = 3100
triggerall = power >= 1000
triggerall = statetype != A
triggerall = RoundState = 2
trigger1 = command = "HwaJai" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "HwaJai" && AILevel = 0
trigger2 = hitdefattr = SC, NA, SA, HA
trigger2 = stateno != [3050,3100)
trigger2 = movecontact
trigger3 = Random < 0.5*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X = [80,160]
trigger3 = p2bodydist Y = 0 || p2bodydist Y = [40,80]
trigger3 = ctrl && p2statetype != L

;---------------------------------------------------------------------------
;Puncture
[State -1, Puncture]
type = ChangeState
value = 3000
triggerall = power >= 1000
triggerall = statetype != A
triggerall = RoundState = 2
trigger1 = command = "Puncture" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "Puncture" && AILevel = 0
trigger2 = hitdefattr = SC, NA, SA, HA
trigger2 = stateno != [3000,3050)
trigger2 = movecontact
trigger3 = Random < 0.5*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X = [80,160]
trigger3 = p2bodydist Y = 0 || p2bodydist Y = [40,80]
trigger3 = ctrl && p2statetype != L

;===========================================================================
;This is not a move, but it sets up var(1) to be 1 if conditions are right
;for a combo into a special move (used below).
;Since a lot of special moves rely on the same conditions, this reduces
;redundant logic.
[State -1, Combo condition Reset]
type = VarSet
trigger1 = 1
var(1) = 0

[State -1, Combo condition Check]
type = VarSet
trigger1 = statetype != A
trigger1 = ctrl
trigger2 = (stateno = [200,299]) || (stateno = [400,499])
trigger2 = stateno != 440 ;Except for sweep kick
trigger2 = movecontact
var(1) = 1

;---------------------------------------------------------------------------


;--------------------------------------------
[State -1, kick]
type = changestate
value = 1060
triggerall = AILevel = 0 || RoundState = 2
triggerall = var(1)
trigger1 = command = "kick" && AILevel = 0
trigger2 = Random < 1*AILevel*AILevel && AILevel != 0
trigger2 = p2bodydist X + p2bodydist Y < 120
trigger3 = Random < 10*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y < 40
trigger3 = stateno = 1060 && MoveHit



;Blender Edge
[State -1, Blender Edge]
type = ChangeState
value = 1010
triggerall = AILevel = 0 || RoundState = 2
triggerall = var(1)
trigger1 = command = "BlenderEdge" && AILevel = 0
trigger2 = Random < 1*AILevel*AILevel && AILevel != 0
trigger2 = p2bodydist X + p2bodydist Y < 120
trigger3 = Random < 10*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y < 40
trigger3 = stateno = 210 && MoveHit

;---------------------------------------------------------------------------
;Light Kung Fu Palm
;Jt[˂ij
[State -1, Light Kung Fu Palm]
type = ChangeState
value = 1000
triggerall = RoundState < 3
triggerall = (command="light kung fu palm") & AILevel = 0
triggerall = Statetype != s
triggerall = Statetype = A
trigger1 = ctrl
;===========================================================================

;---------------------------------------------------------------------------
;Kung Fu Throw
;
[State -1, Kung Fu Throw]
type = ChangeState
value = 800
triggerall = statetype = S
triggerall = stateno != 100
triggerall = p2bodydist X < 5
triggerall = (p2statetype = S) || (p2statetype = C)
triggerall = p2movetype != H
triggerall = ctrl
trigger1 = command = "y"
trigger1 = command = "holdfwd" || command = "holdback"
trigger1 = AILevel = 0
trigger2 = Random < 10*AILevel*AILevel && AILevel != 0

;===========================================================================
;---------------------------------------------------------------------------
;Stand Light Punch
;p`
[State -1, Stand Light Punch]
type = ChangeState
value = 200
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype != A
trigger1 = command = "x" && command != "holddown" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "x" && command != "holddown" && AILevel = 0
trigger2 = stateno = 200 && time > 6
trigger3 = Random < 5*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y < 40
trigger3 = ctrl
trigger4 = Random < 10*AILevel*AILevel && AILevel != 0
trigger4 = p2bodydist X + p2bodydist Y < 40
trigger4 = stateno = 200 && time > 6

;---------------------------------------------------------------------------
;Stand Strong Punch
;p`
[State -1, Stand Strong Punch]
type = ChangeState
value = 210
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype != A
trigger1 = command = "y" && command != "holddown" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "y" && command != "holddown" && AILevel = 0
trigger2 = (stateno = 200) && time > 5
trigger3 = command = "y" && command != "holddown" && AILevel = 0
trigger3 = (stateno = 230) && time > 6
trigger4 = Random < 2*AILevel*AILevel && AILevel != 0
trigger4 = p2bodydist X + p2bodydist Y < 80
trigger4 = ctrl
trigger5 = Random < 10*AILevel*AILevel && AILevel != 0
trigger5 = p2bodydist X + p2bodydist Y < 80
trigger5 = (stateno = 200) && time > 5 && movecontact
trigger6 = Random < 10*AILevel*AILevel && AILevel != 0
trigger6 = p2bodydist X + p2bodydist Y < 80
trigger6 = (stateno = 230) && time > 6 && movecontact

;---------------------------------------------------------------------------
;Stand Light Kick
;LbN
[State -1, Stand Light Kick]
type = ChangeState
value = 230
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype != A
trigger1 = command = "a" && command != "holddown" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "a" && command != "holddown" && AILevel = 0
trigger2 = stateno = 230 && time > 6
trigger3 = Random < 5*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y < 40
trigger3 = ctrl
trigger4 = Random < 10*AILevel*AILevel && AILevel != 0
trigger4 = p2bodydist X + p2bodydist Y < 40
trigger4 = stateno = 230 && time > 6

;---------------------------------------------------------------------------
;Standing Strong Kick
;LbN
[State -1, Standing Strong Kick]
type = ChangeState
value = 240
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype != A
trigger1 = command = "b" && command != "holddown" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "b" && command != "holddown" && AILevel = 0
trigger2 = (stateno = 200) && time > 5
trigger3 = command = "b" && command != "holddown" && AILevel = 0
trigger3 = (stateno = 230) && time > 6
trigger4 = Random < 1*AILevel*AILevel && AILevel != 0
trigger4 = p2bodydist X + p2bodydist Y < 56
trigger4 = ctrl
trigger5 = Random < 3*AILevel*AILevel && AILevel != 0
trigger5 = p2bodydist X + p2bodydist Y < 56
trigger5 = (stateno = 200) && time > 5 && movecontact
trigger6 = Random < 3*AILevel*AILevel && AILevel != 0
trigger6 = p2bodydist X + p2bodydist Y < 56
trigger6 = (stateno = 230) && time > 6 && movecontact

;---------------------------------------------------------------------------
;Taunt
;
[State -1, Taunt]
type = ChangeState
value = 195
triggerall = statetype != A
trigger1 = command = "start" && AILevel = 0
trigger1 = ctrl
trigger2 = Random < 10*AILevel && AILevel != 0
trigger2 = ctrl && RoundState != 2
trigger2 = PrevStateNo != 195

;---------------------------------------------------------------------------
;AI Jump
[State -1, AI Jump]
type = ChangeState
value = 40
triggerall = statetype = S
triggerall = AILevel != 0
triggerall = ctrl
triggerall = Random < 5*AILevel*AILevel
trigger1 = enemynear,statetype = A && enemynear,movetype != H
trigger1 = enemynear,vel y < 0

;---------------------------------------------------------------------------
;Crouching Light Punch
;Ⴊݎp`
[State -1, Crouching Light Punch]
type = ChangeState
value = 400
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype != A
trigger1 = command = "x" && command = "holddown" && AILevel = 0
trigger1 = ctrl
trigger2 = Random < 3*AILevel*AILevel && AILevel != 0
trigger2 = p2bodydist X + p2bodydist Y < 48
trigger2 = ctrl

;---------------------------------------------------------------------------
;Crouching Strong Punch
;Ⴊ݋p`
[State -1, Crouching Strong Punch]
type = ChangeState
value = 410
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype != A
trigger1 = command = "y" && command = "holddown" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "y" && command = "holddown" && AILevel = 0
trigger2 = (stateno = 400) || (stateno = 430)
trigger2 = (time > 9) || (movecontact && time > 5)
trigger2 = Random < 3*AILevel*AILevel && AILevel != 0
trigger2 = p2bodydist X + p2bodydist Y < 64
trigger2 = ctrl

;---------------------------------------------------------------------------
;Crouching Light Kick
;ႪݎLbN
[State -1, Crouching Light Kick]
type = ChangeState
value = 430
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype != A
trigger1 = command = "a" && command = "holddown" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "a" && command = "holddown" && AILevel = 0
trigger2 = (stateno = 400) || (stateno = 430)
trigger2 = (time > 9) || (movecontact && time > 5)
trigger3 = Random < 3*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y < 48
trigger3 = ctrl

;---------------------------------------------------------------------------
;Crouching Strong Kick
;Ⴊ݋LbN
[State -1, Crouching Strong Kick]
type = ChangeState
value = 440
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype != A
trigger1 = command = "b" && command = "holddown" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "b" && command = "holddown" && AILevel = 0
trigger2 = (stateno = 400) || (stateno = 430)
trigger2 = (time > 9) || (movecontact && time > 5)
trigger3 = Random < 4*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y < 64
trigger3 = ctrl

;---------------------------------------------------------------------------
;Jump Light Punch
;󒆎p`
[State -1, Jump Light Punch]
type = ChangeState
value = 600
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype = A
trigger1 = command = "x" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "x" && AILevel = 0
trigger2 = stateno = 1350 ;Air blocking
trigger3 = Random < 7*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y < 64
trigger3 = ctrl

;---------------------------------------------------------------------------
;Jump Strong Punch
[State -1, Jump Strong Punch]
type = ChangeState
value = 610
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype = A
trigger1 = command = "y" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "y" && AILevel = 0
trigger2 = stateno = 600 || stateno = 630 ;jump_x or jump_a
trigger2 = movecontact
trigger3 = command = "y" && AILevel = 0
trigger3 = stateno = 1350 ;Air blocking
trigger4 = Random < 3*AILevel*AILevel && AILevel != 0
trigger4 = p2bodydist X + p2bodydist Y < 64
trigger4 = ctrl
trigger5 = Random < 10*AILevel*AILevel && AILevel != 0
trigger5 = stateno = 600 || stateno = 630
trigger5 = movecontact

;---------------------------------------------------------------------------
;Jump Light Kick
[State -1, Jump Light Kick]
type = ChangeState
value = 630
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype = A
trigger1 = command = "a" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "a" && AILevel = 0
trigger2 = stateno = 1350 ;Air blocking
trigger3 = Random < 7*AILevel*AILevel && AILevel != 0
trigger3 = p2bodydist X + p2bodydist Y < 64
trigger3 = ctrl

;---------------------------------------------------------------------------
;Jump Strong Kick
;󒆋LbN
[State -1, Jump Strong Kick]
type = ChangeState
value = 640
triggerall = AILevel = 0 || RoundState = 2
triggerall = statetype = A
trigger1 = command = "b" && AILevel = 0
trigger1 = ctrl
trigger2 = command = "b" && AILevel = 0
trigger2 = stateno = 600 || stateno = 630 ;jump_x or jump_a
trigger2 = movecontact
trigger3 = command = "b" && AILevel = 0
trigger3 = stateno = 1350 ;Air blocking
trigger4 = Random < 3*AILevel*AILevel && AILevel != 0
trigger4 = p2bodydist X + p2bodydist Y < 64
trigger4 = ctrl
trigger5 = Random < 10*AILevel*AILevel && AILevel != 0
trigger5 = stateno = 600 || stateno = 630
trigger5 = movecontact
